"""Time-related Sphinx directives."""  # noqa: A005

from docutils import nodes
from sphinx.transforms.post_transforms import SphinxPostTransform
from sphinx.util.docutils import SphinxDirective
from sphinx_design.shared import create_component, is_component


class TimeDirective(SphinxDirective):
    """The TimeDirective directive is used to generate a time block."""

    has_content = False
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True

    def run(self):
        time = create_component(
            "ou-time", classes=["ou-time"], children=[nodes.Text(self.arguments[0], self.arguments[0])]
        )
        return [time]


class TimeLatexTransform(SphinxPostTransform):
    """Transform time containers into the LaTeX specific AST structures."""

    default_priority = 199
    formats = ("latex",)

    def run(self):
        """Run the transform"""
        document: nodes.document = self.document
        for node in document.findall(lambda node: is_component(node, "ou-time")):
            new_nodes = [
                nodes.raw("", r"\outime{", format="latex"),
                *node.children,
                nodes.raw("", r"}", format="latex"),
            ]
            node.replace_self(new_nodes)


def setup(app):
    """Setup the Time extensions."""
    app.add_directive("time", TimeDirective)
    app.add_post_transform(TimeLatexTransform)



project = "OU Book Theme"
author = "Mark Hall"
root_doc = "index"
extensions = ["myst_parser", "ou_book_theme", "sphinx_external_toc", "sphinxcontrib.mermaid"]
language = "en"
project_copyright = "2023-%Y"

numfig = True

html_title = "OU Book Theme"
html_theme = "ou_book_theme"

myst_enable_extensions = [
    "amsmath",
    "attrs_inline",
    "colon_fence",
    "deflist",
    "dollarmath",
    "fieldlist",
    "html_admonition",
    "html_image",
    "replacements",
    "smartquotes",
    "strikethrough",
    "substitution",
    "tasklist",
]

latex_toplevel_sectioning = "chapter"
latex_elements = {
    "maketitle": r"\maketitle",
    "preamble": r"\usepackage{ou-book-theme}",
    "sphinxsetup": """OuterLinkColor={RGB}{171,92,74},%
        InnerLinkColor={RGB}{171,92,74},%
        pre_background-TeXcolor={RGB}{255,255,255},%
        pre_border-radius=0pt,%
        pre_padding-top=0.8em,%
        pre_padding-bottom=0.8em,%
        pre_border-left-width=0pt,%
        pre_border-right-width=0pt,%
""",
}



"""JupyterBook CLI extension."""

import os
from importlib import resources
from subprocess import run
from threading import Thread
from typing import Annotated

from livereload import Server
from typer import Argument, Option, Typer

cli = Typer()


@cli.command()
def build(
    path: Annotated[str, Argument(help="Path to the OU book")],
    output_format: Annotated[str, Option(help="The format to generate")] = "html",
) -> None:
    """Serve the OU Book locally."""
    if output_format == "html":
        run(  # noqa:S603
            ["sphinx-build", "--builder", "html", "--fresh-env", path, os.path.join(path, "_build", "html")],  # noqa:S607
            check=False,
        )
    elif output_format == "pdf":
        run(  # noqa:S603
            ["sphinx-build", "--builder", "latex", "--fresh-env", path, os.path.join(path, "_build", "latex")],  # noqa:S607
            check=False,
        )
        if not os.path.exists(os.path.join(path, "_build", "latex")):
            os.makedirs(os.path.join(path, "_build", "latex"), exist_ok=True)
        for resource in resources.files("ou_book_theme").joinpath("latex").iterdir():
            with resources.as_file(resource) as resource_path:
                with open(resource_path, "rb") as in_f:
                    with open(os.path.join(path, "_build", "latex", os.path.basename(str(resource))), "wb") as out_f:
                        out_f.write(in_f.read())
        tex_filename = None
        for filename in os.listdir(os.path.join(path, "_build", "latex")):
            if filename.endswith(".tex"):
                tex_filename = filename
                break
        run(["xelatex", tex_filename], cwd=os.path.join(path, "_build", "latex"), check=False)  # noqa: S603 S607


@cli.command()
def serve(
    path: Annotated[str, Argument(help="Path to the OU book")],
    host: Annotated[str, Option(help="The host to serve the book at")] = "127.0.0.1",
    port: Annotated[int, Option(help="The port to serve the book at")] = 8000,
) -> None:
    """Serve the OU Book locally."""

    def partial_build():
        run(  # noqa:S603
            ["sphinx-build", "--builder", "html", path, os.path.join(path, "_build", "html")],  # noqa:S607
            check=False,
        )

    def full_build():
        run(  # noqa:S603
            ["sphinx-build", "--builder", "html", "--fresh-env", path, os.path.join(path, "_build", "html")],  # noqa:S607
            check=False,
        )

    initial_build = Thread(target=full_build)
    initial_build.start()

    server = Server()
    server.watch(f"{path}/**/*.md", partial_build)
    server.watch(f"{path}/**/*.yml", full_build)
    server.watch(f"{path}/**/*.py", full_build)
    server.watch(f"{path}/**/*.png", full_build)
    server.watch(f"{path}/**/*.jpg", full_build)
    server.watch(f"{path}/**/*.jpeg", full_build)
    server.watch(f"{path}/**/*.svg", full_build)
    server.serve(root=f"{path}/_build/html", port=port, host=host)
